package jp.gihyo.rest.clock;

import java.util.HashMap;
import java.util.Map;

import jp.gihyo.rest.clock.util.FtlHelper;
import jp.gihyo.rest.clock.util.TimeZoneRepository;

import org.restlet.Context;
import org.restlet.Router;
import org.restlet.data.Form;
import org.restlet.data.MediaType;
import org.restlet.data.Reference;
import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.data.Status;
import org.restlet.resource.Representation;
import org.restlet.resource.Resource;
import org.restlet.resource.Variant;

public class TimeZoneListResource extends Resource {

    public static void attachTo(Router router) {
        router.attach("", TimeZoneListResource.class);
    }

    public TimeZoneListResource(Context context, Request request, Response response) {
        super(context, request, response);
        getVariants().add(new Variant(MediaType.APPLICATION_XHTML_XML));
    }

    @Override
    public boolean allowGet() {
        return true;
    }

    @Override
    public Representation getRepresentation(Variant variant) {
        MediaType mediaType = variant.getMediaType();
        if (MediaType.APPLICATION_XHTML_XML.equals(mediaType)) {
            Map<String, Object> binding = new HashMap<String, Object>();
            binding.put("timeZoneIds", TimeZoneRepository.listIds());
            binding.put("rootRef", getRequest().getRootRef());
            return FtlHelper.asXhtml("timeZoneList.ftl", binding);
        }
        throw new IllegalStateException("cannot be happen");
    }

    @Override
    public boolean allowPost() {
        return true;
    }

    @Override
    public void post(Representation entity) {
        if (entity.getMediaType().equals(MediaType.APPLICATION_WWW_FORM, true)) {
            Form form = new Form(entity);
            String customId = form.getFirstValue("customId");

            Reference rootRef = getRequest().getRootRef();
            String targetUri = rootRef + "/" + customId;
            Response resp = getResponse();
            Map<String, Object> binding = new HashMap<String, Object>();
            binding.put("rootRef", getRequest().getRootRef());
            binding.put("customId", customId);

            if (TimeZoneRepository.contains(customId)) {
                resp.setEntity(FtlHelper.asXhtml("customTimeZoneConflicted.ftl", binding));
                // Locationヘッダを設定するためにRestletではredirectSeeOtherメソッドを使う。一種のバッドノウハウ。
                resp.redirectSeeOther(targetUri);
                resp.setStatus(Status.CLIENT_ERROR_CONFLICT);
                return;
            }

            boolean registered = TimeZoneRepository.register(customId);
            if (registered) {
                resp.setEntity(FtlHelper.asXhtml("customTimeZoneCreated.ftl", binding));
                // Locationヘッダを設定する
                resp.redirectSeeOther(targetUri);
                resp.setStatus(Status.SUCCESS_CREATED);
            } else {
                resp.setEntity(FtlHelper.asXhtml("customTimeZoneBadRequest.ftl", binding));
                resp.setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
            }
        }
    }

}