package jp.gihyo.rest.clock;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.TimeZone;

import jp.gihyo.rest.clock.util.ClockWrapper;
import jp.gihyo.rest.clock.util.FtlHelper;
import jp.gihyo.rest.clock.util.TimeZoneRepository;

import org.restlet.Context;
import org.restlet.Router;
import org.restlet.data.MediaType;
import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.data.Status;
import org.restlet.resource.Representation;
import org.restlet.resource.Resource;
import org.restlet.resource.StringRepresentation;
import org.restlet.resource.Variant;

public class WorldTimeResource extends Resource {

    public static void attachTo(Router router) {
        router.attach("/{timezone}", WorldTimeResource.class);
        router.attach("/{area}/{location}", WorldTimeResource.class);
    }

    private TimeZone timeZone;

    public WorldTimeResource(Context context, Request request, Response response) {
        super(context, request, response);

        String timeZoneId = getTimeZoneIdFrom(request);
        if (TimeZoneRepository.contains(timeZoneId)) {
            this.timeZone = TimeZoneRepository.find(timeZoneId);
            // XHTMLの優先度を上にする
            getVariants().add(new Variant(MediaType.APPLICATION_XHTML_XML));
            getVariants().add(new Variant(MediaType.TEXT_PLAIN));
        } else {
            getResponse().setStatus(Status.CLIENT_ERROR_NOT_FOUND);
        }
    }

    private String getTimeZoneIdFrom(Request request) {
        Map<String, Object> attrs = request.getAttributes();
        String tzId = (String) attrs.get("timezone");
        String area = (String) attrs.get("area");
        String location = (String) attrs.get("location");

        String timeZoneId = null;
        if (location != null) {
            timeZoneId = area + "/" + location;
        } else if (tzId != null) {
            timeZoneId = tzId;
        }
        return timeZoneId;
    }

    @Override
    public boolean allowGet() {
        return true;
    }

    @Override
    public Representation getRepresentation(Variant variant) {
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ");
        df.setTimeZone(this.timeZone);
        String currentTime = df.format(now());

        MediaType mediaType = variant.getMediaType();
        if (MediaType.TEXT_PLAIN.equals(mediaType)) {
            return new StringRepresentation(currentTime);
        } else if (MediaType.APPLICATION_XHTML_XML.equals(mediaType)) {
            Map<String, Object> binding = new HashMap<String, Object>();
            binding.put("timeZone", this.timeZone);
            binding.put("currentTime", currentTime);
            binding.put("rootRef", getRequest().getRootRef());
            return FtlHelper.asXhtml("timeZone.ftl", binding);
        }
        throw new IllegalStateException("cannot be happen");
    }

    private Date now() {
        return ClockWrapper.now();
    }
}
