package jp.gihyo.rest;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.restlet.Application;
import org.restlet.Component;
import org.restlet.Context;
import org.restlet.Restlet;
import org.restlet.Router;
import org.restlet.data.MediaType;
import org.restlet.data.Protocol;
import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.resource.Representation;
import org.restlet.resource.Resource;
import org.restlet.resource.StringRepresentation;
import org.restlet.resource.Variant;

import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebResponse;

public class RestletStackTest {

    public static class HelloWorldResource extends Resource {
        public HelloWorldResource(Context context, Request request,
                Response response) {
            super(context, request, response);
            getVariants().add(new Variant(MediaType.TEXT_PLAIN));
        }

        @Override
        public boolean allowGet() {
            return true;
        }

        @Override
        public Representation getRepresentation(Variant variant) {
            if (MediaType.TEXT_PLAIN.equals(variant.getMediaType())) {
                return new StringRepresentation("Hello REST World!");
            }
            throw new IllegalStateException("cannot be happen");
        }
    }

    private Component component;

    @Before
    public void コンポーネント準備() throws Exception {
        component = new Component();
        // ポート8182でHTTPサーバを起動する設定を行う
        component.getServers().add(Protocol.HTTP, 8182);

        Application app = new Application(component.getContext()) {
            @Override
            public Restlet createRoot() {
                Router router = new Router(getContext());
                // アプリケーション以下の"/hw"にリソースをマッピング
                router.attach("/hw", HelloWorldResource.class);
                return router;
            }
        };

        // ローカルホストの"/sample"にアプリケーションをマッピング
        component.getDefaultHost().attach("/sample", app);

        // テスト用のサーバを開始する
        component.start();
    }

    @After
    public void コンポーネント終了処理() throws Exception {
        component.stop();
    }

    @Test
    public void HttpUnitでリソースにGETアクセスする() throws Exception {
        GetMethodWebRequest req = new GetMethodWebRequest(
                "http://localhost:8182/sample/hw");
        WebConversation wc = new WebConversation();
        WebResponse resp = wc.getResource(req);

        // サーバからの応答をassertする
        assertThat(resp.getResponseCode(), is(200));
        assertThat(resp.getText(), is("Hello REST World!"));
    }
}
