package jp.gihyo.rest.first;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.restlet.Client;
import org.restlet.Restlet;
import org.restlet.Server;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Protocol;
import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.resource.Representation;


public class HelloRestletTest {
    private Server server;

    @Before
    public void 組み込みサーバ準備() throws Exception {
        // "Hello Restlet"と返すだけのRestletを作成
        Restlet restlet = new Restlet() {
            @Override
            public void handle(Request request, Response response) {
                response.setEntity("Hello Restlet!", MediaType.TEXT_PLAIN);
            }
        };
        // ポート8182でHTTPサーバを起動し、Restletを登録
        server = new Server(Protocol.HTTP, 8182, restlet);
        server.start();
    }

    @After
    public void 組み込みサーバ停止() throws Exception {
        if (server != null) {
            server.stop();
        }
    }

    @Test
    public void RestletClientを使用した最小構成を試す() throws Exception {
        // サーバに投げるGETリクエストを作成
        Request request = new Request(Method.GET, "http://localhost:8182");

        // RestletのHTTPクライアントコネクタでサーバに接続
        Client client = new Client(Protocol.HTTP);
        Response response = client.handle(request);

        // サーバからの応答を検証する
        Representation output = response.getEntity();
        assertThat(output.getText(), is("Hello Restlet!"));
    }
}
